from gbox_sdk._client import GboxClient
from gbox_sdk.wrapper.box.base import BaseBox
from gbox_sdk.types.v1.linux_box import LinuxBox


class LinuxBoxOperator(BaseBox):
    def __init__(self, client: GboxClient, data: LinuxBox):
        super().__init__(client, data)



from typing import List, Union

from gbox_sdk._client import GboxClient
from gbox_sdk.types.v1.linux_box import LinuxBox
from gbox_sdk.wrapper.box.action import ActionOperator
from gbox_sdk.wrapper.box.browser import BrowserOperator
from gbox_sdk.types.v1.android_box import AndroidBox
from gbox_sdk.wrapper.box.file_system import FileSystemOperator
from gbox_sdk.types.v1.box_stop_params import BoxStopParams
from gbox_sdk.types.v1.box_start_params import BoxStartParams
from gbox_sdk.types.v1.box_run_code_params import BoxRunCodeParams
from gbox_sdk.types.v1.box_terminate_params import BoxTerminateParams
from gbox_sdk.types.v1.box_live_view_url_params import BoxLiveViewURLParams
from gbox_sdk.types.v1.box_live_view_url_response import BoxLiveViewURLResponse
from gbox_sdk.types.v1.box_execute_commands_params import BoxExecuteCommandsParams
from gbox_sdk.types.v1.box_web_terminal_url_params import BoxWebTerminalURLParams
from gbox_sdk.types.v1.box_web_terminal_url_response import BoxWebTerminalURLResponse


class BaseBox:
    def __init__(self, client: GboxClient, data: Union[LinuxBox, AndroidBox]):
        self.client = client
        self.data = data

        self.action = ActionOperator(self.client, self.data.id)
        self.fs = FileSystemOperator(self.client, self.data.id)
        self.browser = BrowserOperator(self.client, self.data.id)

    def _sync_data(self) -> None:
        res = self.client.v1.boxes.retrieve(box_id=self.data.id)
        self.data = res

    def start(self, body: BoxStartParams) -> "BaseBox":
        self.client.v1.boxes.start(box_id=self.data.id, **body)
        self._sync_data()
        return self

    def stop(self, body: BoxStopParams) -> "BaseBox":
        self.client.v1.boxes.stop(box_id=self.data.id, **body)
        self._sync_data()
        return self

    def terminate(self, body: BoxTerminateParams) -> "BaseBox":
        self.client.v1.boxes.terminate(box_id=self.data.id, **body)
        self._sync_data()
        return self

    def command(self, body: Union[BoxExecuteCommandsParams, str, List[str]]) -> "BaseBox":
        if isinstance(body, str):
            body = BoxExecuteCommandsParams(commands=[body])
        elif isinstance(body, list):
            body = BoxExecuteCommandsParams(commands=body)
        self.client.v1.boxes.execute_commands(box_id=self.data.id, **body)
        self._sync_data()
        return self

    def run_code(self, body: Union[BoxRunCodeParams, str]) -> "BaseBox":
        if isinstance(body, str):
            body = BoxRunCodeParams(code=body)
        self.client.v1.boxes.run_code(box_id=self.data.id, **body)
        self._sync_data()
        return self

    def live_view(self, body: BoxLiveViewURLParams) -> BoxLiveViewURLResponse:
        return self.client.v1.boxes.live_view_url(box_id=self.data.id, **body)

    def web_terminal(self, body: BoxWebTerminalURLParams) -> BoxWebTerminalURLResponse:
        return self.client.v1.boxes.web_terminal_url(box_id=self.data.id, **body)



from typing import List, Union, Mapping, Optional

import httpx

from gbox_sdk import GboxClient
from gbox_sdk._types import NOT_GIVEN, Timeout, NotGiven
from gbox_sdk.wrapper.utils import is_linux_box, is_android_box
from gbox_sdk.wrapper.box.linux import LinuxBoxOperator
from gbox_sdk.types.v1.linux_box import LinuxBox
from gbox_sdk.types.v1.android_box import AndroidBox
from gbox_sdk.types.v1.box_list_params import BoxListParams
from gbox_sdk.types.v1.box_list_response import BoxListResponse
from gbox_sdk.wrapper.box.android.android import AndroidBoxOperator
from gbox_sdk.types.v1.box_terminate_params import BoxTerminateParams
from gbox_sdk.types.v1.box_retrieve_response import BoxRetrieveResponse
from gbox_sdk.types.v1.box_create_linux_params import BoxCreateLinuxParams
from gbox_sdk.types.v1.box_create_android_params import BoxCreateAndroidParams

BoxOperator = Union[AndroidBoxOperator, LinuxBoxOperator]


class BoxListOperatorResponse:
    def __init__(
        self,
        operators: List[BoxOperator],
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        total: Optional[int] = None,
    ):
        self.operators = operators
        self.page = page
        self.page_size = page_size
        self.total = total


class GboxSDK:
    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[Union[str, httpx.URL]] = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: Optional[int] = None,
        default_headers: Optional[Mapping[str, str]] = None,
        default_query: Optional[Mapping[str, object]] = None,
        http_client: Optional[httpx.Client] = None,
        _strict_response_validation: Optional[bool] = None,
    ):
        self.client = GboxClient(
            api_key=api_key,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries if max_retries is not None else 2,
            default_headers=default_headers,
            default_query=default_query,
            http_client=http_client,
            _strict_response_validation=_strict_response_validation
            if _strict_response_validation is not None
            else False,
        )

    def create_android(self, body: BoxCreateAndroidParams) -> AndroidBoxOperator:
        res = self.client.v1.boxes.create_android(**body)
        return AndroidBoxOperator(self.client, res)

    def create_linux(self, body: BoxCreateLinuxParams) -> LinuxBoxOperator:
        res = self.client.v1.boxes.create_linux(**body)
        return LinuxBoxOperator(self.client, res)

    def list_info(self, query: Optional[BoxListParams] = None) -> BoxListResponse:
        if query is None:
            query = BoxListParams()
        return self.client.v1.boxes.list(**query)

    def list(self, query: Optional[BoxListParams] = None) -> BoxListOperatorResponse:
        if query is None:
            query = BoxListParams()
        res = self.client.v1.boxes.list(**query)
        data = getattr(res, "data", [])
        operators = [self.data_to_operator(item) for item in data]
        return BoxListOperatorResponse(
            operators=operators,
            page=getattr(res, "page", None),
            page_size=getattr(res, "page_size", None),
            total=getattr(res, "total", None),
        )

    def get_info(self, box_id: str) -> BoxRetrieveResponse:
        return self.client.v1.boxes.retrieve(box_id)

    def get(self, box_id: str) -> BoxOperator:
        res = self.client.v1.boxes.retrieve(box_id)
        return self.data_to_operator(res)

    def terminate(self, box_id: str, body: Optional[BoxTerminateParams] = None) -> None:
        if body is None:
            body = BoxTerminateParams()
        self.client.v1.boxes.terminate(box_id, **body)

    def data_to_operator(self, data: Union[AndroidBox, LinuxBox]) -> BoxOperator:
        if is_android_box(data):
            android_box: AndroidBox = AndroidBox(**data.model_dump())
            return AndroidBoxOperator(self.client, android_box)
        elif is_linux_box(data):
            linux_box: LinuxBox = LinuxBox(**data.model_dump())
            return LinuxBoxOperator(self.client, linux_box)
        else:
            raise ValueError(f"Invalid box type: {data.type}")

    def get_client(self) -> GboxClient:
        return self.client



from gbox_sdk._client import GboxClient
from gbox_sdk.wrapper.box.base import BaseBox
from gbox_sdk.types.v1.android_box import AndroidBox
from gbox_sdk.wrapper.box.android.app_manager import AndroidAppManager
from gbox_sdk.wrapper.box.android.pkg_manager import AndroidPkgManager


class AndroidBoxOperator(BaseBox):
    def __init__(self, client: GboxClient, data: AndroidBox):
        super().__init__(client, data)
        self.app = AndroidAppManager(client, data)
        self.pkg = AndroidPkgManager(client, data)



import os
import base64
from typing import Optional

from gbox_sdk._client import GboxClient
from gbox_sdk.types.v1.boxes.action_drag_params import ActionDragParams
from gbox_sdk.types.v1.boxes.action_move_params import ActionMoveParams
from gbox_sdk.types.v1.boxes.action_type_params import ActionTypeParams
from gbox_sdk.types.v1.boxes.action_click_params import ActionClickParams
from gbox_sdk.types.v1.boxes.action_swipe_params import SwipeSimple, SwipeAdvanced
from gbox_sdk.types.v1.boxes.action_touch_params import ActionTouchParams
from gbox_sdk.types.v1.boxes.action_drag_response import ActionDragResponse
from gbox_sdk.types.v1.boxes.action_move_response import ActionMoveResponse
from gbox_sdk.types.v1.boxes.action_scroll_params import ActionScrollParams
from gbox_sdk.types.v1.boxes.action_type_response import ActionTypeResponse
from gbox_sdk.types.v1.boxes.action_click_response import ActionClickResponse
from gbox_sdk.types.v1.boxes.action_swipe_response import ActionSwipeResponse
from gbox_sdk.types.v1.boxes.action_touch_response import ActionTouchResponse
from gbox_sdk.types.v1.boxes.action_scroll_response import ActionScrollResponse
from gbox_sdk.types.v1.boxes.action_press_key_params import ActionPressKeyParams
from gbox_sdk.types.v1.boxes.action_screenshot_params import ActionScreenshotParams
from gbox_sdk.types.v1.boxes.action_press_key_response import ActionPressKeyResponse
from gbox_sdk.types.v1.boxes.action_press_button_params import ActionPressButtonParams
from gbox_sdk.types.v1.boxes.action_screenshot_response import ActionScreenshotResponse
from gbox_sdk.types.v1.boxes.action_press_button_response import ActionPressButtonResponse
from gbox_sdk.types.v1.boxes.action_screen_rotation_params import ActionScreenRotationParams
from gbox_sdk.types.v1.boxes.action_screen_rotation_response import ActionScreenRotationResponse


class ActionScreenshot(ActionScreenshotParams, total=False):
    path: Optional[str]


class ActionOperator:
    def __init__(self, client: GboxClient, box_id: str):
        self.client = client
        self.box_id = box_id

    def click(self, body: ActionClickParams) -> ActionClickResponse:
        return self.client.v1.boxes.actions.click(box_id=self.box_id, **body)

    def drag(self, body: ActionDragParams) -> ActionDragResponse:
        return self.client.v1.boxes.actions.drag(box_id=self.box_id, **body)

    def swipe_simple(self, body: SwipeSimple) -> ActionSwipeResponse:
        return self.client.v1.boxes.actions.swipe(box_id=self.box_id, **body)

    def swipe_advanced(self, body: SwipeAdvanced) -> ActionSwipeResponse:
        return self.client.v1.boxes.actions.swipe(box_id=self.box_id, **body)

    def press_key(self, body: ActionPressKeyParams) -> ActionPressKeyResponse:
        return self.client.v1.boxes.actions.press_key(box_id=self.box_id, **body)

    def press_button(self, body: ActionPressButtonParams) -> ActionPressButtonResponse:
        return self.client.v1.boxes.actions.press_button(box_id=self.box_id, **body)

    def move(self, body: ActionMoveParams) -> ActionMoveResponse:
        return self.client.v1.boxes.actions.move(box_id=self.box_id, **body)

    def scroll(self, body: ActionScrollParams) -> ActionScrollResponse:
        return self.client.v1.boxes.actions.scroll(box_id=self.box_id, **body)

    def touch(self, body: ActionTouchParams) -> ActionTouchResponse:
        return self.client.v1.boxes.actions.touch(box_id=self.box_id, **body)

    def type(self, body: ActionTypeParams) -> ActionTypeResponse:
        return self.client.v1.boxes.actions.type(box_id=self.box_id, **body)

    def screenshot(self, body: ActionScreenshotParams) -> ActionScreenshotResponse:
        if body.get("output_format") is None:
            body["output_format"] = "base64"
        return self.client.v1.boxes.actions.screenshot(box_id=self.box_id, **body)

    def screen_rotation(self, body: ActionScreenRotationParams) -> ActionScreenRotationResponse:
        return self.client.v1.boxes.actions.screen_rotation(box_id=self.box_id, **body)

    def save_data_url_to_file(self, data_url: str, file_path: str) -> None:
        if not data_url.startswith("data:"):
            raise ValueError("Invalid data URL format")
        parts = data_url.split(",")
        if len(parts) != 2:
            raise ValueError("Invalid data URL format")
        base64_data = parts[1]

        dir_path = os.path.dirname(file_path)
        if dir_path and not os.path.exists(dir_path):
            os.makedirs(dir_path, exist_ok=True)

        with open(file_path, "wb") as f:
            f.write(base64.b64decode(base64_data))



from gbox_sdk._client import GboxClient


class BrowserOperator:
    def __init__(self, client: GboxClient, box_id: str):
        self.client = client
        self.box_id = box_id

    def cdpUrl(self) -> str:
        return self.client.v1.boxes.browser.cdp_url(box_id=self.box_id)



from typing import List, Union, Optional

from gbox_sdk._client import GboxClient
from gbox_sdk.types.v1.boxes.f_info_params import FInfoParams
from gbox_sdk.types.v1.boxes.f_list_params import FListParams
from gbox_sdk.types.v1.boxes.f_read_params import FReadParams
from gbox_sdk.types.v1.boxes.f_write_params import WriteFile, WriteFileByBinary
from gbox_sdk.types.v1.boxes.f_exists_params import FExistsParams
from gbox_sdk.types.v1.boxes.f_list_response import Data, DataDir, DataFile, FListResponse
from gbox_sdk.types.v1.boxes.f_read_response import FReadResponse
from gbox_sdk.types.v1.boxes.f_remove_params import FRemoveParams
from gbox_sdk.types.v1.boxes.f_rename_params import FRenameParams
from gbox_sdk.types.v1.boxes.f_write_response import FWriteResponse
from gbox_sdk.types.v1.boxes.f_exists_response import FExistsResponse
from gbox_sdk.types.v1.boxes.f_rename_response import FRenameResponse


class FileSystemOperator:
    def __init__(self, client: GboxClient, box_id: str):
        self.client = client
        self.box_id = box_id

    def list_info(self, body: Union[FListParams, str]) -> FListResponse:
        if isinstance(body, str):
            return self.client.v1.boxes.fs.list(box_id=self.box_id, path=body)
        else:
            return self.client.v1.boxes.fs.list(box_id=self.box_id, **body)

    def list(self, body: Union[FListParams, str]) -> List[Union["FileOperator", "DirectoryOperator"]]:
        res = self.list_info(body)
        return [self.data_to_operator(r) for r in res.data]

    def read(self, body: FReadParams) -> FReadResponse:
        return self.client.v1.boxes.fs.read(box_id=self.box_id, **body)

    def write_text(self, body: WriteFile) -> FWriteResponse:
        return self.client.v1.boxes.fs.write(box_id=self.box_id, **body)

    def write_binary(self, body: WriteFileByBinary) -> FWriteResponse:
        return self.client.v1.boxes.fs.write(box_id=self.box_id, **body)

    def remove(self, body: FRemoveParams) -> None:
        self.client.v1.boxes.fs.remove(box_id=self.box_id, **body)
        return

    def exists(self, body: FExistsParams) -> FExistsResponse:
        return self.client.v1.boxes.fs.exists(box_id=self.box_id, **body)

    def rename(self, body: FRenameParams) -> FRenameResponse:
        return self.client.v1.boxes.fs.rename(box_id=self.box_id, **body)

    def get(self, body: FInfoParams) -> Union["FileOperator", "DirectoryOperator"]:
        res = self.client.v1.boxes.fs.info(box_id=self.box_id, **body)
        if res.type == "file":
            data_file = DataFile(
                path=res.path, type="file", mode=res.mode, name=res.name, size=res.size, lastModified=res.last_modified
            )
            return FileOperator(self.client, self.box_id, data_file)
        else:
            data_dir = DataDir(path=res.path, type="dir", mode=res.mode, name=res.name, lastModified=res.last_modified)
            return DirectoryOperator(self.client, self.box_id, data_dir)

    def data_to_operator(self, data: Optional[Data]) -> Union["FileOperator", "DirectoryOperator"]:
        if data is None:
            raise ValueError("data is None")
        if data.type == "file":
            return FileOperator(self.client, self.box_id, data)
        else:
            return DirectoryOperator(self.client, self.box_id, data)


class FileOperator:
    def __init__(self, client: GboxClient, box_id: str, data: DataFile):
        self.client = client
        self.box_id = box_id
        self.data = data

    def write_text(self, body: WriteFile) -> FWriteResponse:
        params = WriteFile(
            path=self.data.path, content=body.get("content", ""), working_dir=body.get("working_dir") or ""
        )
        return self.client.v1.boxes.fs.write(box_id=self.box_id, **params)

    def write_binary(self, body: WriteFileByBinary) -> FWriteResponse:
        params = WriteFileByBinary(
            path=self.data.path, content=body.get("content", b""), working_dir=body.get("working_dir") or ""
        )
        return self.client.v1.boxes.fs.write(box_id=self.box_id, **params)

    def read(self, body: Optional[FReadParams] = None) -> FReadResponse:
        if body is None:
            body = FReadParams(path=self.data.path, working_dir="")
        return self.client.v1.boxes.fs.read(box_id=self.box_id, **body)

    def rename(self, body: FRenameParams) -> FRenameResponse:
        params = FRenameParams(
            old_path=self.data.path, new_path=body.get("new_path", ""), working_dir=body.get("working_dir") or ""
        )
        return self.client.v1.boxes.fs.rename(box_id=self.box_id, **params)


class DirectoryOperator:
    def __init__(self, client: GboxClient, box_id: str, data: DataDir):
        self.client = client
        self.box_id = box_id
        self.data = data

    def list_info(self, body: Optional[FListParams] = None) -> FListResponse:
        if body is None:
            body = FListParams(path=self.data.path)
        return self.client.v1.boxes.fs.list(box_id=self.box_id, **body)

    def list(self, body: Optional[FListParams] = None) -> List[Union["FileOperator", "DirectoryOperator"]]:
        res = self.list_info(body)
        result: List[Union["FileOperator", "DirectoryOperator"]] = []
        for r in res.data:
            if r.type == "file":
                file = DataFile(
                    path=r.path, type=r.type, mode=r.mode, name=r.name, size=r.size, lastModified=r.last_modified
                )
                result.append(FileOperator(self.client, self.box_id, file))
            else:
                dir = DataDir(path=r.path, type=r.type, mode=r.mode, name=r.name, lastModified=r.last_modified)
                result.append(DirectoryOperator(self.client, self.box_id, dir))
        return result

    def rename(self, body: FRenameParams) -> FRenameResponse:
        params = FRenameParams(
            old_path=self.data.path, new_path=body.get("new_path", ""), working_dir=body.get("working_dir") or ""
        )
        return self.client.v1.boxes.fs.rename(box_id=self.box_id, **params)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ActionTouchParams", "Point", "PointStart"]


class ActionTouchParams(TypedDict, total=False):
    points: Required[Iterable[Point]]
    """Array of touch points and their actions"""

    include_screenshot: Annotated[bool, PropertyInfo(alias="includeScreenshot")]
    """Whether to include screenshots in the action response.

    If false, the screenshot object will still be returned but with empty URIs.
    Default is false.
    """

    output_format: Annotated[Literal["base64", "storageKey"], PropertyInfo(alias="outputFormat")]
    """Type of the URI. default is base64."""

    screenshot_delay: Annotated[str, PropertyInfo(alias="screenshotDelay")]
    """Delay after performing the action, before taking the final screenshot.

    Execution flow:

    1. Take screenshot before action
    2. Perform the action
    3. Wait for screenshotDelay (this parameter)
    4. Take screenshot after action

    Example: '500ms' means wait 500ms after the action before capturing the final
    screenshot.

    Supported time units: ms (milliseconds), s (seconds), m (minutes), h (hours)
    Example formats: "500ms", "30s", "5m", "1h" Default: 500ms Maximum allowed: 30s
    """


class PointStart(TypedDict, total=False):
    x: Required[float]
    """Starting X coordinate"""

    y: Required[float]
    """Starting Y coordinate"""


class Point(TypedDict, total=False):
    start: Required[PointStart]
    """Initial touch point position"""

    actions: Iterable[object]
    """Sequence of actions to perform after initial touch"""
